@@ -5,6 +5,8 @@ from rest_framework.response import Response
 from rest_framework_simplejwt.views import TokenObtainPairView, TokenRefreshView
 from rest_framework_simplejwt.serializers import TokenRefreshSerializer
 
+from rest_routes.permissions import can_suspend_user_perm
+
 
 """Third Party Imports"""
 from rest_api_payload import success_response, error_response
@@ -19,7 +21,9 @@ from rest_routes.serializers import (
     OTPSerializer,
     ResendOTPSerializer,
     ResetPasswordOTPSerializer,
+    SuspendUserSerializer,
     UserLoginObtainPairSerializer,
+    UserSerializer,
 )
 from rest_routes.otp_verifications import OTPVerification
 
@@ -35,7 +39,7 @@ otp_verify = OTPVerification()
 User = get_user_model()
 
 
-class Konnichiwa(views.APIView):
+class Hello(views.APIView):
 
     PROTOCOL = "http://"
 
@@ -53,17 +57,21 @@ class Konnichiwa(views.APIView):
                 "confirm_otp": BASE_URL + "rest_routes/confirm_otp/",
                 "resend_otp_code": BASE_URL + "rest_routes/resend_otp_code/",
                 "logout": BASE_URL + "rest_routes/logout/",
-                "change_password": BASE_URL + "rest_routes/change_password/<str:email>/",
-                "reset_password_otp (otp)": BASE_URL + "rest_routes/password_reset_otp/",
-                "reset_password_otp_confirm (otp)": BASE_URL + "rest_routes/password_reset_otp/confirm/",
-                "reset_password_otp_complete (otp)": BASE_URL + "rest_routes/password_reset_otp/complete/",
+                "change_password": BASE_URL
+                + "rest_routes/change_password/<str:email>/",
+                "reset_password_otp (otp)": BASE_URL
+                + "rest_routes/password_reset_otp/",
+                "reset_password_otp_confirm (otp)": BASE_URL
+                + "rest_routes/password_reset_otp/confirm/",
+                "reset_password_otp_complete (otp)": BASE_URL
+                + "rest_routes/password_reset_otp/complete/",
                 "suspend_user": BASE_URL + "rest_routes/suspend_user/<str:email>/",
             },
         }
         return Response(data=welcome_data, status=status.HTTP_200_OK)
 
 
-class RegisterOniichan(views.APIView):
+class RegisterUser(views.APIView):
     serializer_class = RegisterUserSerializer
     permission_classes = [AllowAny]
 
@@ -91,19 +99,19 @@ class RegisterOniichan(views.APIView):
         return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class LoginOniichan(TokenObtainPairView):
+class LoginUser(TokenObtainPairView):
     """Inherits TokenObtainPairView from rest_framework simplejwt"""
 
     serializer_class = UserLoginObtainPairSerializer
 
 
-class RefreshLoginOniichan(TokenRefreshView):
+class RefreshLoginUser(TokenRefreshView):
     """Inherits TokenRefreshView from rest_framework simplejwt"""
 
     serializer_class = TokenRefreshSerializer
 
 
-class ConfirmOniichanOTP(views.APIView):
+class ConfirmUserOTP(views.APIView):
     serializer_class = OTPSerializer
     permission_classes = [AllowAny]
 
@@ -172,14 +180,14 @@ class ConfirmOniichanOTP(views.APIView):
                 return Response(data=payload, status=status.HTTP_200_OK)
 
 
-class ResendOniichanOTP(views.APIView):
+class ResendUserOTP(views.APIView):
     serializer_class = ResendOTPSerializer
     permission_classes = [AllowAny]
-    
-    def get_user(self, email:str):
+
+    def get_user(self, email: str):
         """
         Get a user by email
-        
+
         :param email: The email of the user you want to get
         :type email: str
         :return: A user object
@@ -237,94 +245,90 @@ class ResendOniichanOTP(views.APIView):
                 return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-# class SuspendUserApiView(views.APIView):
-#     permissions_classes = [IsAuthenticated]
-
-#     def get_single_user(self, email:str) -> Response:
-
-#         try:
-#             user = User.objects.get(is_active=True, email=email)
-#             return user
-#         except User.DoesNotExist:
-#             payload = error_response(
-#                 status="failed",
-#                 message="User does not exist!"
-#             )
-#             return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
-
-#     def get(self, request:HttpRequest, email:str) -> Response:
-#         user = self.get_single_user(email=email)
-#         serializer = SuspendUserSerializer(user)
-
-#         payload = success_response(
-#             status="success",
-#             message="User retrieved!",
-#             data=serializer.data
-#         )
-#         return Response(data=payload, status=status.HTTP_200_OK)
-
-#     def put(self, request:HttpRequest, email:str) -> Response:
-#         user = self.get_single_user(email=email)
-#         serializer = SuspendUserSerializer(data=request.data, instance=user)
-
-#         # Serialized data from the serializer
-#         serialized_email = serializer.initial_data.get("email")
-#         serialized_active_flag = serializer.initial_data.get("is_staff")
-
-#         # Logic to check if the user request has the following perms
-#         # -> is_staff, is_active, is_authenticated, has_checker_perm
-#         user_has_checker_perm = has_controller_perm_func(request.user)
-
-#         # Check if the serializer is valid and
-#         # the user making the request has controller permission
-#         if serializer.is_valid() == user_has_checker_perm is True:
-
-#             # Checks if the serialized is_active flag is set to True and;
-#             # the serialized email is equal to the user email
-#             if serialized_active_flag is True and serialized_email is user.email:
-
-#                 # Suspend the user
-#                 user.is_active = False
-#                 user.save()
-
-#                 # Custom payload
-#                 payload = success_response(
-#                     status="success",
-#                     message="User successfully suspended!",
-#                     data=serializer.data
-#                 )
-#                 return Response(data=payload, status=status.HTTP_202_ACCEPTED)
-
-#             # Checks if the serialized is_active flag is set to False and;
-#             # the serialized email is equal to the user email
-#             elif serialized_active_flag is False and serialized_email is user.email:
-
-#                 # Activate the user
-#                 user.is_active = True
-#                 user.save()
-
-#                 # Custom payload
-#                 payload = success_response(
-#                     status="success",
-#                     message="User successfully activated!",
-#                     data=serializer.data
-#                 )
-#                 return Response(data=payload, status=status.HTTP_202_ACCEPTED)
-
-#             payload = error_response(
-#                 success="failed",
-#                 message="User does not have the required permission to perform this action!"
-#             )
-#             return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
-
-#         payload = error_response(
-#             status="failed",
-#             message="Something went wrong. Please try again!"
-#         )
-#         return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
+class SuspendUser(views.APIView):
+    permissions_classes = [IsAuthenticated]
+    user_serializer = UserSerializer
+    suspend_user_serializer = SuspendUserSerializer
+    
+    def get_single_user(self, email:str) -> Response:
+        
+        try:
+            user = User.objects.get(email=email)
+            return user
+        except User.DoesNotExist:
+            payload = error_response(
+                status="error",
+                message="User does not exist!"
+            )
+            return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
+        
+    def get(self, request:HttpRequest, email:str) -> Response:
+        user = self.get_single_user(email=email)
+        serializer = self.user_serializer(user)
+        
+        payload = success_response(
+            status="success",
+            message="User retrieved!",
+            data=serializer.data
+        )
+        return Response(data=payload, status=status.HTTP_200_OK)
+    
+    def put(self, request:HttpRequest, email:str) -> Response:
+        user = self.get_single_user(email=email)
+        serializer = self.suspend_user_serializer(user, data=request.data)
+        
+        """
+        Check if the serializer is valid and the user 
+        making the request has permission to suspend a user
+        """
+        if serializer.is_valid() == True and can_suspend_user_perm(request=request) == True:
+            
+            is_active = serializer.validated_data.get("is_active")
+            
+            if is_active == False:
+                
+                user.is_active = False
+                user.save()
+            
+                payload = success_response(
+                    status="success",
+                    message="User has been suspended!",
+                    data={
+                        "user": self.user_serializer(user).data,
+                        "user_meta": serializer.data
+                    }
+                )
+                return Response(data=payload, status=status.HTTP_202_ACCEPTED)
+            
+            elif is_active == True:
+                
+                user.is_active = True
+                user.save()
+                
+                payload = success_response(
+                    status="success",
+                    message="User has been activated!",
+                    data=serializer.data
+                )
+                return Response(data=payload, status=status.HTTP_202_ACCEPTED)
+        
+        elif can_suspend_user_perm(request=request) == False:
+            
+            payload = error_response(
+                status="error",
+                message="You don't have the required permission to perform this action!"
+            )
+            return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
+          
+        else:  
+            payload = error_response(
+                status="error",
+                message=serializer.errors
+            )
+            return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class ChangeOniichanPassword(views.APIView):
+class ChangeUserPassword(views.APIView):
     permissions_classes = [IsAuthenticated]
     serializer_class = ChangeUserPasswordSerializer
 
@@ -402,7 +406,7 @@ class ChangeOniichanPassword(views.APIView):
         return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class ResetOniichanPasswordOTPAPIView(views.APIView):
+class ResetUserPasswordOTPAPIView(views.APIView):
     permission_classes = [AllowAny]
     serializer_class = ResetPasswordOTPSerializer
 
@@ -436,7 +440,7 @@ class ResetOniichanPasswordOTPAPIView(views.APIView):
             return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class ConfirmResetOniichanPasswordOTPAPIView(views.APIView):
+class ConfirmResetUserPasswordOTPAPIView(views.APIView):
     permission_classes = [AllowAny]
     serializer_class = OTPSerializer
 
@@ -488,7 +492,7 @@ class ConfirmResetOniichanPasswordOTPAPIView(views.APIView):
                 return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class ResetOniichanPasswordOTPCompleteAPIView(views.APIView):
+class ResetUserPasswordOTPCompleteAPIView(views.APIView):
     permission_classes = [AllowAny]
     serializer_class = CompleteResetPasswordOTPSerializer
 
@@ -531,17 +535,15 @@ class ResetOniichanPasswordOTPCompleteAPIView(views.APIView):
                     message="Password incorrect. Plelase try again!",
                 )
                 return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
-        
+
         else:
 
             """Return a response message to the user with the error message"""
-            payload = error_response(
-                status="failed", message=serializer.errors
-            )
+            payload = error_response(status="failed", message=serializer.errors)
             return Response(data=payload, status=status.HTTP_400_BAD_REQUEST)
 
 
-class LogOniichanOut(views.APIView):
+class LogUserOut(views.APIView):
     """
     Removes the authenticated user's ID from the request and flushes their
     session data.
@@ -557,9 +559,7 @@ class LogOniichanOut(views.APIView):
         """Wipes user request"""
         logout(request)
 
-        payload = success_response(
-            status="success", message="You logged out!", data={}
-        )
+        payload = success_response(status="success", message="You logged out!", data={})
         return Response(data=payload, status=status.HTTP_204_NO_CONTENT)
 
 
